"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.copySync = exports.copy = void 0;
// Copyright 2018-2024 the Deno authors. All rights reserved. MIT license.
const dntShim = __importStar(require("../../../../../_dnt.shims.js"));
const basename_js_1 = require("../../path/0.225.1/basename.js");
const join_js_1 = require("../../path/0.225.1/join.js");
const resolve_js_1 = require("../../path/0.225.1/resolve.js");
const ensure_dir_js_1 = require("./ensure_dir.js");
const assert_js_1 = require("../../assert/0.225.2/assert.js");
const _get_file_info_type_js_1 = require("./_get_file_info_type.js");
const _to_path_string_js_1 = require("./_to_path_string.js");
const _is_subdir_js_1 = require("./_is_subdir.js");
const isWindows = dntShim.Deno.build.os === "windows";
async function ensureValidCopy(src, dest, options) {
    let destStat;
    try {
        destStat = await dntShim.Deno.lstat(dest);
    }
    catch (err) {
        if (err instanceof dntShim.Deno.errors.NotFound) {
            return;
        }
        throw err;
    }
    if (options.isFolder && !destStat.isDirectory) {
        throw new Error(`Cannot overwrite non-directory '${dest}' with directory '${src}'.`);
    }
    if (!options.overwrite) {
        throw new dntShim.Deno.errors.AlreadyExists(`'${dest}' already exists.`);
    }
    return destStat;
}
function ensureValidCopySync(src, dest, options) {
    let destStat;
    try {
        destStat = dntShim.Deno.lstatSync(dest);
    }
    catch (err) {
        if (err instanceof dntShim.Deno.errors.NotFound) {
            return;
        }
        throw err;
    }
    if (options.isFolder && !destStat.isDirectory) {
        throw new Error(`Cannot overwrite non-directory '${dest}' with directory '${src}'.`);
    }
    if (!options.overwrite) {
        throw new dntShim.Deno.errors.AlreadyExists(`'${dest}' already exists.`);
    }
    return destStat;
}
/* copy file to dest */
async function copyFile(src, dest, options) {
    await ensureValidCopy(src, dest, options);
    await dntShim.Deno.copyFile(src, dest);
    if (options.preserveTimestamps) {
        const statInfo = await dntShim.Deno.stat(src);
        (0, assert_js_1.assert)(statInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(statInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        await dntShim.Deno.utime(dest, statInfo.atime, statInfo.mtime);
    }
}
/* copy file to dest synchronously */
function copyFileSync(src, dest, options) {
    ensureValidCopySync(src, dest, options);
    dntShim.Deno.copyFileSync(src, dest);
    if (options.preserveTimestamps) {
        const statInfo = dntShim.Deno.statSync(src);
        (0, assert_js_1.assert)(statInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(statInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        dntShim.Deno.utimeSync(dest, statInfo.atime, statInfo.mtime);
    }
}
/* copy symlink to dest */
async function copySymLink(src, dest, options) {
    await ensureValidCopy(src, dest, options);
    const originSrcFilePath = await dntShim.Deno.readLink(src);
    const type = (0, _get_file_info_type_js_1.getFileInfoType)(await dntShim.Deno.lstat(src));
    if (isWindows) {
        await dntShim.Deno.symlink(originSrcFilePath, dest, {
            type: type === "dir" ? "dir" : "file",
        });
    }
    else {
        await dntShim.Deno.symlink(originSrcFilePath, dest);
    }
    if (options.preserveTimestamps) {
        const statInfo = await dntShim.Deno.lstat(src);
        (0, assert_js_1.assert)(statInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(statInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        await dntShim.Deno.utime(dest, statInfo.atime, statInfo.mtime);
    }
}
/* copy symlink to dest synchronously */
function copySymlinkSync(src, dest, options) {
    ensureValidCopySync(src, dest, options);
    const originSrcFilePath = dntShim.Deno.readLinkSync(src);
    const type = (0, _get_file_info_type_js_1.getFileInfoType)(dntShim.Deno.lstatSync(src));
    if (isWindows) {
        dntShim.Deno.symlinkSync(originSrcFilePath, dest, {
            type: type === "dir" ? "dir" : "file",
        });
    }
    else {
        dntShim.Deno.symlinkSync(originSrcFilePath, dest);
    }
    if (options.preserveTimestamps) {
        const statInfo = dntShim.Deno.lstatSync(src);
        (0, assert_js_1.assert)(statInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(statInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        dntShim.Deno.utimeSync(dest, statInfo.atime, statInfo.mtime);
    }
}
/* copy folder from src to dest. */
async function copyDir(src, dest, options) {
    const destStat = await ensureValidCopy(src, dest, {
        ...options,
        isFolder: true,
    });
    if (!destStat) {
        await (0, ensure_dir_js_1.ensureDir)(dest);
    }
    if (options.preserveTimestamps) {
        const srcStatInfo = await dntShim.Deno.stat(src);
        (0, assert_js_1.assert)(srcStatInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(srcStatInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        await dntShim.Deno.utime(dest, srcStatInfo.atime, srcStatInfo.mtime);
    }
    src = (0, _to_path_string_js_1.toPathString)(src);
    dest = (0, _to_path_string_js_1.toPathString)(dest);
    const promises = [];
    for await (const entry of dntShim.Deno.readDir(src)) {
        const srcPath = (0, join_js_1.join)(src, entry.name);
        const destPath = (0, join_js_1.join)(dest, (0, basename_js_1.basename)(srcPath));
        if (entry.isSymlink) {
            promises.push(copySymLink(srcPath, destPath, options));
        }
        else if (entry.isDirectory) {
            promises.push(copyDir(srcPath, destPath, options));
        }
        else if (entry.isFile) {
            promises.push(copyFile(srcPath, destPath, options));
        }
    }
    await Promise.all(promises);
}
/* copy folder from src to dest synchronously */
function copyDirSync(src, dest, options) {
    const destStat = ensureValidCopySync(src, dest, {
        ...options,
        isFolder: true,
    });
    if (!destStat) {
        (0, ensure_dir_js_1.ensureDirSync)(dest);
    }
    if (options.preserveTimestamps) {
        const srcStatInfo = dntShim.Deno.statSync(src);
        (0, assert_js_1.assert)(srcStatInfo.atime instanceof Date, `statInfo.atime is unavailable`);
        (0, assert_js_1.assert)(srcStatInfo.mtime instanceof Date, `statInfo.mtime is unavailable`);
        dntShim.Deno.utimeSync(dest, srcStatInfo.atime, srcStatInfo.mtime);
    }
    src = (0, _to_path_string_js_1.toPathString)(src);
    dest = (0, _to_path_string_js_1.toPathString)(dest);
    for (const entry of dntShim.Deno.readDirSync(src)) {
        const srcPath = (0, join_js_1.join)(src, entry.name);
        const destPath = (0, join_js_1.join)(dest, (0, basename_js_1.basename)(srcPath));
        if (entry.isSymlink) {
            copySymlinkSync(srcPath, destPath, options);
        }
        else if (entry.isDirectory) {
            copyDirSync(srcPath, destPath, options);
        }
        else if (entry.isFile) {
            copyFileSync(srcPath, destPath, options);
        }
    }
}
/**
 * Asynchronously copy a file or directory. The directory can have contents.
 * Like `cp -r`.
 *
 * If `src` is a directory it will copy everything inside of this directory,
 * not the entire directory itself. If `src` is a file, `dest` cannot be a
 * directory.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param src The source file/directory path as a string or URL.
 * @param dest The destination file/directory path as a string or URL.
 * @param options Options for copying.
 * @returns A promise that resolves once the copy operation completes.
 *
 * @example Basic usage
 * ```ts
 * import { copy } from "@std/fs/copy";
 *
 * await copy("./foo", "./bar");
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` without
 * overwriting.
 *
 * @example Overwriting files/directories
 * ```ts
 * import { copy } from "@std/fs/copy";
 *
 * await copy("./foo", "./bar", { overwrite: true });
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` and overwrite
 * any existing files or directories.
 *
 * @example Preserving timestamps
 * ```ts
 * import { copy } from "@std/fs/copy";
 *
 * await copy("./foo", "./bar", { preserveTimestamps: true });
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` and set the
 * last modification and access times to the ones of the original source files.
 */
async function copy(src, dest, options = {}) {
    src = (0, resolve_js_1.resolve)((0, _to_path_string_js_1.toPathString)(src));
    dest = (0, resolve_js_1.resolve)((0, _to_path_string_js_1.toPathString)(dest));
    if (src === dest) {
        throw new Error("Source and destination cannot be the same.");
    }
    const srcStat = await dntShim.Deno.lstat(src);
    if (srcStat.isDirectory && (0, _is_subdir_js_1.isSubdir)(src, dest)) {
        throw new Error(`Cannot copy '${src}' to a subdirectory of itself, '${dest}'.`);
    }
    if (srcStat.isSymlink) {
        await copySymLink(src, dest, options);
    }
    else if (srcStat.isDirectory) {
        await copyDir(src, dest, options);
    }
    else if (srcStat.isFile) {
        await copyFile(src, dest, options);
    }
}
exports.copy = copy;
/**
 * Synchronously copy a file or directory. The directory can have contents.
 * Like `cp -r`.
 *
 * If `src` is a directory it will copy everything inside of this directory,
 * not the entire directory itself. If `src` is a file, `dest` cannot be a
 * directory.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param src The source file/directory path as a string or URL.
 * @param dest The destination file/directory path as a string or URL.
 * @param options Options for copying.
 * @returns A void value that returns once the copy operation completes.
 *
 * @example Basic usage
 * ```ts
 * import { copySync } from "@std/fs/copy";
 *
 * copySync("./foo", "./bar");
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` without
 * overwriting.
 *
 * @example Overwriting files/directories
 * ```ts
 * import { copySync } from "@std/fs/copy";
 *
 * copySync("./foo", "./bar", { overwrite: true });
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` and overwrite
 * any existing files or directories.
 *
 * @example Preserving timestamps
 * ```ts
 * import { copySync } from "@std/fs/copy";
 *
 * copySync("./foo", "./bar", { preserveTimestamps: true });
 * ```
 *
 * This will copy the file or directory at `./foo` to `./bar` and set the
 * last modification and access times to the ones of the original source files.
 */
function copySync(src, dest, options = {}) {
    src = (0, resolve_js_1.resolve)((0, _to_path_string_js_1.toPathString)(src));
    dest = (0, resolve_js_1.resolve)((0, _to_path_string_js_1.toPathString)(dest));
    if (src === dest) {
        throw new Error("Source and destination cannot be the same.");
    }
    const srcStat = dntShim.Deno.lstatSync(src);
    if (srcStat.isDirectory && (0, _is_subdir_js_1.isSubdir)(src, dest)) {
        throw new Error(`Cannot copy '${src}' to a subdirectory of itself, '${dest}'.`);
    }
    if (srcStat.isSymlink) {
        copySymlinkSync(src, dest, options);
    }
    else if (srcStat.isDirectory) {
        copyDirSync(src, dest, options);
    }
    else if (srcStat.isFile) {
        copyFileSync(src, dest, options);
    }
}
exports.copySync = copySync;
