"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureSymlinkSync = exports.ensureSymlink = void 0;
// Copyright 2018-2024 the Deno authors. All rights reserved. MIT license.
const dntShim = __importStar(require("../../../../../_dnt.shims.js"));
const dirname_js_1 = require("../../path/0.225.1/dirname.js");
const resolve_js_1 = require("../../path/0.225.1/resolve.js");
const ensure_dir_js_1 = require("./ensure_dir.js");
const _get_file_info_type_js_1 = require("./_get_file_info_type.js");
const _to_path_string_js_1 = require("./_to_path_string.js");
const isWindows = dntShim.Deno.build.os === "windows";
function resolveSymlinkTarget(target, linkName) {
    if (typeof target !== "string")
        return target; // URL is always absolute path
    if (typeof linkName === "string") {
        return (0, resolve_js_1.resolve)((0, dirname_js_1.dirname)(linkName), target);
    }
    else {
        return new URL(target, linkName);
    }
}
/**
 * Asynchronously ensures that the link exists, and points to a valid file. If
 * the directory structure does not exist, it is created. If the link already
 * exists, it is not modified but error is thrown if it is not point to the
 * given target.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param target The source file path as a string or URL.
 * @param linkName The destination link path as a string or URL.
 * @returns A void promise that resolves once the link exists.
 *
 * @example
 * ```ts
 * import { ensureSymlink } from "@std/fs/ensure-symlink";
 *
 * await ensureSymlink("./folder/targetFile.dat", "./folder/targetFile.link.dat");
 * ```
 */
async function ensureSymlink(target, linkName) {
    const targetRealPath = resolveSymlinkTarget(target, linkName);
    const srcStatInfo = await dntShim.Deno.lstat(targetRealPath);
    const srcFilePathType = (0, _get_file_info_type_js_1.getFileInfoType)(srcStatInfo);
    await (0, ensure_dir_js_1.ensureDir)((0, dirname_js_1.dirname)((0, _to_path_string_js_1.toPathString)(linkName)));
    const options = isWindows
        ? {
            type: srcFilePathType === "dir" ? "dir" : "file",
        }
        : undefined;
    try {
        await dntShim.Deno.symlink(target, linkName, options);
    }
    catch (error) {
        if (!(error instanceof dntShim.Deno.errors.AlreadyExists)) {
            throw error;
        }
        const linkStatInfo = await dntShim.Deno.lstat(linkName);
        if (!linkStatInfo.isSymlink) {
            const type = (0, _get_file_info_type_js_1.getFileInfoType)(linkStatInfo);
            throw new dntShim.Deno.errors.AlreadyExists(`A '${type}' already exists at the path: ${linkName}`);
        }
        const linkPath = await dntShim.Deno.readLink(linkName);
        const linkRealPath = (0, resolve_js_1.resolve)(linkPath);
        if (linkRealPath !== targetRealPath) {
            throw new dntShim.Deno.errors.AlreadyExists(`A symlink targeting to an undesired path already exists: ${linkName} -> ${linkRealPath}`);
        }
    }
}
exports.ensureSymlink = ensureSymlink;
/**
 * Synchronously ensures that the link exists, and points to a valid file. If
 * the directory structure does not exist, it is created. If the link already
 * exists, it is not modified but error is thrown if it is not point to the
 * given target.
 *
 * Requires the `--allow-read` and `--allow-write` flag.
 *
 * @param target The source file path as a string or URL.
 * @param linkName The destination link path as a string or URL.
 * @returns A void value that returns once the link exists.
 *
 * @example
 * ```ts
 * import { ensureSymlinkSync } from "@std/fs/ensure-symlink";
 *
 * ensureSymlinkSync("./folder/targetFile.dat", "./folder/targetFile.link.dat");
 * ```
 */
function ensureSymlinkSync(target, linkName) {
    const targetRealPath = resolveSymlinkTarget(target, linkName);
    const srcStatInfo = dntShim.Deno.lstatSync(targetRealPath);
    const srcFilePathType = (0, _get_file_info_type_js_1.getFileInfoType)(srcStatInfo);
    (0, ensure_dir_js_1.ensureDirSync)((0, dirname_js_1.dirname)((0, _to_path_string_js_1.toPathString)(linkName)));
    const options = isWindows
        ? {
            type: srcFilePathType === "dir" ? "dir" : "file",
        }
        : undefined;
    try {
        dntShim.Deno.symlinkSync(target, linkName, options);
    }
    catch (error) {
        if (!(error instanceof dntShim.Deno.errors.AlreadyExists)) {
            throw error;
        }
        const linkStatInfo = dntShim.Deno.lstatSync(linkName);
        if (!linkStatInfo.isSymlink) {
            const type = (0, _get_file_info_type_js_1.getFileInfoType)(linkStatInfo);
            throw new dntShim.Deno.errors.AlreadyExists(`A '${type}' already exists at the path: ${linkName}`);
        }
        const linkPath = dntShim.Deno.readLinkSync(linkName);
        const linkRealPath = (0, resolve_js_1.resolve)(linkPath);
        if (linkRealPath !== targetRealPath) {
            throw new dntShim.Deno.errors.AlreadyExists(`A symlink targeting to an undesired path already exists: ${linkName} -> ${linkRealPath}`);
        }
    }
}
exports.ensureSymlinkSync = ensureSymlinkSync;
