"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.expandGlobSync = exports.expandGlob = void 0;
// Copyright 2018-2024 the Deno authors. All rights reserved. MIT license.
const dntShim = __importStar(require("../../../../../_dnt.shims.js"));
const glob_to_regexp_js_1 = require("../../path/0.225.1/glob_to_regexp.js");
const join_globs_js_1 = require("../../path/0.225.1/join_globs.js");
const is_glob_js_1 = require("../../path/0.225.1/is_glob.js");
const is_absolute_js_1 = require("../../path/0.225.1/is_absolute.js");
const resolve_js_1 = require("../../path/0.225.1/resolve.js");
const constants_js_1 = require("../../path/0.225.1/constants.js");
const walk_js_1 = require("./walk.js");
const assert_js_1 = require("../../assert/0.225.2/assert.js");
const _to_path_string_js_1 = require("./_to_path_string.js");
const _create_walk_entry_js_1 = require("./_create_walk_entry.js");
const isWindows = dntShim.Deno.build.os === "windows";
function split(path) {
    const s = constants_js_1.SEPARATOR_PATTERN.source;
    const segments = path
        .replace(new RegExp(`^${s}|${s}$`, "g"), "")
        .split(constants_js_1.SEPARATOR_PATTERN);
    const isAbsolute_ = (0, is_absolute_js_1.isAbsolute)(path);
    return {
        segments,
        isAbsolute: isAbsolute_,
        hasTrailingSep: !!path.match(new RegExp(`${s}$`)),
        winRoot: isWindows && isAbsolute_ ? segments.shift() : undefined,
    };
}
function throwUnlessNotFound(error) {
    if (!(error instanceof dntShim.Deno.errors.NotFound)) {
        throw error;
    }
}
function comparePath(a, b) {
    if (a.path < b.path)
        return -1;
    if (a.path > b.path)
        return 1;
    return 0;
}
/**
 * Returns an async iterator that yields each file path matching the given glob
 * pattern. The file paths are relative to the provided `root` directory.
 * If `root` is not provided, the current working directory is used.
 * The `root` directory is not included in the yielded file paths.
 *
 * Requires the `--allow-read` flag.
 *
 * @param glob The glob pattern to expand.
 * @param options Additional options for the expansion.
 * @returns An async iterator that yields each walk entry matching the glob
 * pattern.
 *
 * @example Basic usage
 *
 * File structure:
 * ```
 * folder
 * ├── script.ts
 * └── foo.ts
 * ```
 *
 * ```ts
 * // script.ts
 * import { expandGlob } from "@std/fs/expand-glob";
 *
 * const entries = [];
 * for await (const entry of expandGlob("*.ts")) {
 *   entries.push(entry);
 * }
 *
 * entries[0]!.path; // "/Users/user/folder/script.ts"
 * entries[0]!.name; // "script.ts"
 * entries[0]!.isFile; // false
 * entries[0]!.isDirectory; // true
 * entries[0]!.isSymlink; // false
 *
 * entries[1]!.path; // "/Users/user/folder/foo.ts"
 * entries[1]!.name; // "foo.ts"
 * entries[1]!.isFile; // true
 * entries[1]!.isDirectory; // false
 * entries[1]!.isSymlink; // false
 * ```
 */
async function* expandGlob(glob, { root, exclude = [], includeDirs = true, extended = true, globstar = true, caseInsensitive, followSymlinks, canonicalize, } = {}) {
    const { segments, isAbsolute: isGlobAbsolute, hasTrailingSep, winRoot, } = split((0, _to_path_string_js_1.toPathString)(glob));
    root ??= isGlobAbsolute ? winRoot ?? "/" : dntShim.Deno.cwd();
    const globOptions = { extended, globstar, caseInsensitive };
    const absRoot = isGlobAbsolute ? root : (0, resolve_js_1.resolve)(root); // root is always string here
    const resolveFromRoot = (path) => (0, resolve_js_1.resolve)(absRoot, path);
    const excludePatterns = exclude
        .map(resolveFromRoot)
        .map((s) => (0, glob_to_regexp_js_1.globToRegExp)(s, globOptions));
    const shouldInclude = (path) => !excludePatterns.some((p) => !!path.match(p));
    let fixedRoot = isGlobAbsolute
        ? winRoot !== undefined ? winRoot : "/"
        : absRoot;
    while (segments.length > 0 && !(0, is_glob_js_1.isGlob)(segments[0])) {
        const seg = segments.shift();
        (0, assert_js_1.assert)(seg !== undefined);
        fixedRoot = (0, join_globs_js_1.joinGlobs)([fixedRoot, seg], globOptions);
    }
    let fixedRootInfo;
    try {
        fixedRootInfo = await (0, _create_walk_entry_js_1.createWalkEntry)(fixedRoot);
    }
    catch (error) {
        return throwUnlessNotFound(error);
    }
    async function* advanceMatch(walkInfo, globSegment) {
        if (!walkInfo.isDirectory) {
            return;
        }
        else if (globSegment === "..") {
            const parentPath = (0, join_globs_js_1.joinGlobs)([walkInfo.path, ".."], globOptions);
            try {
                if (shouldInclude(parentPath)) {
                    return yield await (0, _create_walk_entry_js_1.createWalkEntry)(parentPath);
                }
            }
            catch (error) {
                throwUnlessNotFound(error);
            }
            return;
        }
        else if (globSegment === "**") {
            return yield* (0, walk_js_1.walk)(walkInfo.path, {
                skip: excludePatterns,
                maxDepth: globstar ? Infinity : 1,
                followSymlinks,
                canonicalize,
            });
        }
        const globPattern = (0, glob_to_regexp_js_1.globToRegExp)(globSegment, globOptions);
        for await (const walkEntry of (0, walk_js_1.walk)(walkInfo.path, {
            maxDepth: 1,
            skip: excludePatterns,
            followSymlinks,
        })) {
            if (walkEntry.path !== walkInfo.path &&
                walkEntry.name.match(globPattern)) {
                yield walkEntry;
            }
        }
    }
    let currentMatches = [fixedRootInfo];
    for (const segment of segments) {
        // Advancing the list of current matches may introduce duplicates, so we
        // pass everything through this Map.
        const nextMatchMap = new Map();
        await Promise.all(currentMatches.map(async (currentMatch) => {
            for await (const nextMatch of advanceMatch(currentMatch, segment)) {
                nextMatchMap.set(nextMatch.path, nextMatch);
            }
        }));
        currentMatches = [...nextMatchMap.values()].sort(comparePath);
    }
    if (hasTrailingSep) {
        currentMatches = currentMatches.filter((entry) => entry.isDirectory);
    }
    if (!includeDirs) {
        currentMatches = currentMatches.filter((entry) => !entry.isDirectory);
    }
    yield* currentMatches;
}
exports.expandGlob = expandGlob;
/**
 * Returns an iterator that yields each file path matching the given glob
 * pattern. The file paths are relative to the provided `root` directory.
 * If `root` is not provided, the current working directory is used.
 * The `root` directory is not included in the yielded file paths.
 *
 * Requires the `--allow-read` flag.
 *
 * @param glob The glob pattern to expand.
 * @param options Additional options for the expansion.
 * @returns An iterator that yields each walk entry matching the glob pattern.
 *
 * @example Basic usage
 *
 * File structure:
 * ```
 * folder
 * ├── script.ts
 * └── foo.ts
 * ```
 *
 * ```ts
 * // script.ts
 * import { expandGlobSync } from "@std/fs/expand-glob";
 *
 * const entries = [];
 * for (const entry of expandGlobSync("*.ts")) {
 *   entries.push(entry);
 * }
 *
 * entries[0]!.path; // "/Users/user/folder/script.ts"
 * entries[0]!.name; // "script.ts"
 * entries[0]!.isFile; // false
 * entries[0]!.isDirectory; // true
 * entries[0]!.isSymlink; // false
 *
 * entries[1]!.path; // "/Users/user/folder/foo.ts"
 * entries[1]!.name; // "foo.ts"
 * entries[1]!.isFile; // true
 * entries[1]!.isDirectory; // false
 * entries[1]!.isSymlink; // false
 * ```
 */
function* expandGlobSync(glob, { root, exclude = [], includeDirs = true, extended = true, globstar = true, caseInsensitive, followSymlinks, canonicalize, } = {}) {
    const { segments, isAbsolute: isGlobAbsolute, hasTrailingSep, winRoot, } = split((0, _to_path_string_js_1.toPathString)(glob));
    root ??= isGlobAbsolute ? winRoot ?? "/" : dntShim.Deno.cwd();
    const globOptions = { extended, globstar, caseInsensitive };
    const absRoot = isGlobAbsolute ? root : (0, resolve_js_1.resolve)(root); // root is always string here
    const resolveFromRoot = (path) => (0, resolve_js_1.resolve)(absRoot, path);
    const excludePatterns = exclude
        .map(resolveFromRoot)
        .map((s) => (0, glob_to_regexp_js_1.globToRegExp)(s, globOptions));
    const shouldInclude = (path) => !excludePatterns.some((p) => !!path.match(p));
    let fixedRoot = isGlobAbsolute
        ? winRoot !== undefined ? winRoot : "/"
        : absRoot;
    while (segments.length > 0 && !(0, is_glob_js_1.isGlob)(segments[0])) {
        const seg = segments.shift();
        (0, assert_js_1.assert)(seg !== undefined);
        fixedRoot = (0, join_globs_js_1.joinGlobs)([fixedRoot, seg], globOptions);
    }
    let fixedRootInfo;
    try {
        fixedRootInfo = (0, _create_walk_entry_js_1.createWalkEntrySync)(fixedRoot);
    }
    catch (error) {
        return throwUnlessNotFound(error);
    }
    function* advanceMatch(walkInfo, globSegment) {
        if (!walkInfo.isDirectory) {
            return;
        }
        else if (globSegment === "..") {
            const parentPath = (0, join_globs_js_1.joinGlobs)([walkInfo.path, ".."], globOptions);
            try {
                if (shouldInclude(parentPath)) {
                    return yield (0, _create_walk_entry_js_1.createWalkEntrySync)(parentPath);
                }
            }
            catch (error) {
                throwUnlessNotFound(error);
            }
            return;
        }
        else if (globSegment === "**") {
            return yield* (0, walk_js_1.walkSync)(walkInfo.path, {
                skip: excludePatterns,
                maxDepth: globstar ? Infinity : 1,
                followSymlinks,
                canonicalize,
            });
        }
        const globPattern = (0, glob_to_regexp_js_1.globToRegExp)(globSegment, globOptions);
        for (const walkEntry of (0, walk_js_1.walkSync)(walkInfo.path, {
            maxDepth: 1,
            skip: excludePatterns,
            followSymlinks,
        })) {
            if (walkEntry.path !== walkInfo.path &&
                walkEntry.name.match(globPattern)) {
                yield walkEntry;
            }
        }
    }
    let currentMatches = [fixedRootInfo];
    for (const segment of segments) {
        // Advancing the list of current matches may introduce duplicates, so we
        // pass everything through this Map.
        const nextMatchMap = new Map();
        for (const currentMatch of currentMatches) {
            for (const nextMatch of advanceMatch(currentMatch, segment)) {
                nextMatchMap.set(nextMatch.path, nextMatch);
            }
        }
        currentMatches = [...nextMatchMap.values()].sort(comparePath);
    }
    if (hasTrailingSep) {
        currentMatches = currentMatches.filter((entry) => entry.isDirectory);
    }
    if (!includeDirs) {
        currentMatches = currentMatches.filter((entry) => !entry.isDirectory);
    }
    yield* currentMatches;
}
exports.expandGlobSync = expandGlobSync;
