"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _CommandCache_instances, _CommandCache_fetchCommandOld, _CommandCache_fetchCommand;
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandCache = exports.KNOWN_METHODS = void 0;
const dntShim = __importStar(require("../_dnt.shims.js"));
const deps_js_1 = require("../deps.js");
const CommandsData_js_1 = require("./CommandsData.js");
exports.KNOWN_METHODS = [
    "ask",
    "blend",
    "describe",
    "fast",
    "help",
    "imagine",
    "info",
    "invite",
    "prefer",
    "private",
    "public",
    "relax",
    "settings",
    "show",
    "stealth",
    "subscribe",
];
/**
 * get command from Midjourney discord bot and cache it in memory and disk.
 */
class CommandCache {
    constructor(channel_id, authorization) {
        _CommandCache_instances.add(this);
        Object.defineProperty(this, "channel_id", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: channel_id
        });
        Object.defineProperty(this, "authorization", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: authorization
        });
        Object.defineProperty(this, "cacheDirectory", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "cache", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        this.cacheDirectory = "cmdCache";
    }
    async getCommand(name) {
        const fromCode = CommandsData_js_1.CommandsData.application_commands.find((cmd) => cmd.name === name);
        if (fromCode) {
            return fromCode;
        }
        // try from memory cache
        if (this.cache[name]) {
            return this.cache[name];
        }
        let command;
        // try from disk cache
        dntShim.Deno.mkdirSync(this.cacheDirectory, { recursive: true });
        const cacheFile = this.getcacheFile(name);
        if (cacheFile) {
            try {
                command = JSON.parse(await dntShim.Deno.readTextFile(cacheFile));
            }
            catch (_e) {
                // failed to load command from cache.
            }
        }
        // get from discord
        if (!command) {
            const application_commands = await __classPrivateFieldGet(this, _CommandCache_instances, "m", _CommandCache_fetchCommand).call(this, name);
            if (application_commands[0]) {
                command = application_commands[0];
                if (cacheFile) {
                    await dntShim.Deno.writeTextFile(cacheFile, JSON.stringify(command, undefined, 2));
                }
            }
        }
        // save in memory
        if (command) {
            this.cache[name] = command;
            return command;
        }
        throw Error(`Failed to get application_commands for command ${name}`);
    }
    getcacheFile(name) {
        if (!this.cacheDirectory)
            return "";
        return deps_js_1.path.join(this.cacheDirectory, `${name}.json`);
    }
}
exports.CommandCache = CommandCache;
_CommandCache_instances = new WeakSet(), _CommandCache_fetchCommandOld = 
// Old code
async function _CommandCache_fetchCommandOld(name) {
    deps_js_1.logger.info(`CommandCache: ${name} not in cache, requesting Discord server.`);
    const url = `https://discord.com/api/v9/channels/${this.channel_id}/application-commands/search?type=1&query=${name}&limit=1&include_applications=false`;
    const response = await fetch(url, {
        headers: { authorization: this.authorization },
    });
    const data = (await response.json());
    if ("application_commands" in data) {
        const application_commands = data.application_commands;
        return application_commands;
    }
    return [];
}, _CommandCache_fetchCommand = async function _CommandCache_fetchCommand(name) {
    deps_js_1.logger.info(`CommandCache: ${name} not in cache, requesting Discord server.`);
    const url = `https://discord.com/api/v9/channels/${this.channel_id}/application-command-index`;
    const response = await fetch(url, {
        headers: { authorization: this.authorization },
    });
    const data = (await response.json());
    if (data.application_commands) {
        const application_commands = data.application_commands;
        if (name) {
            const command = application_commands.filter((cmd) => cmd.name === name);
            if (!command.length) {
                deps_js_1.logger.error(`CommandCache: ${name} not found, available cmds: ${application_commands.map((cmd) => cmd.name).join(", ")}`);
            }
            return command;
        }
        return application_commands;
    }
    if (data.code && data.message) {
        deps_js_1.logger.error(`fetchCommand from ${url} failed With error: ${data.message} code: ${data.code}`);
    }
    return [];
};
